package jp.co.secioss.lib;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.Authenticator;
import java.net.HttpURLConnection;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.PasswordAuthentication;
import java.net.Socket;
import java.net.SocketException;
import java.net.URL;
import java.net.UnknownHostException;
import java.security.InvalidKeyException;
import java.security.KeyStore;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Enumeration;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.openauthentication.otp.OneTimePasswordAlgorithm;

public class SeciossCommon {

    private static int TIMEZONE = 12;
    private static final String SECRET_CHARS_TYP0 = "0123456789";
    private static final String SECRET_CHARS_TYP1 = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    private static final String SECRET_CHARS_TYP2 = "!\"#$%&'()-^\\=~|@[`{;:]+*},./\\<>?_";         //!\"#$%&'()*+,-./@[]^_{|}~";

    private static final String PATTERN_URL = "^(http|https)(://)(.+)\\.(.+)";
    private static final String PATTERN_USERID = ".+";    // "^[a-zA-Z]+[0-9]*@(.+)\\.(.+)"

    private static final String PATTERN_IPCONFIG = "[^:]*: ([0-9A-F][0-9A-F](-[0-9A-F][0-9A-F]){5})$";
    private static final String PATTERN_IFCONFIG_MAC = "\\s*ether ([0-9a-f][0-9a-f](:[0-9a-f][0-9a-f]){5})\\s*";
    private static final String PATTERN_IFCONFIG_LNX = "\\s*HWaddr ([0-9a-f][0-9a-f](:[0-9a-f][0-9a-f]){5})\\s*";

    private static final String[] DEFALUT_SKIP_NIC = new String[] {"Juniper","awdl0:", "Virtual"};
    private static final String[] DEFALUT_SKIP_NIC_EXCLUDE = new String[] {"Hyper-V"};

    /**
     * 16進数のStirngをchar配列に変換
     * @param hex 16進数のStirng
     * @return 変換されたchar配列
     */
    public static char[] hexToChar(String hex) {
        //char[] kDigits = "0123456789abcdef".toCharArray();
        int length = hex.length() / 2;
        char[] raw = new char[length];

        for (int i = 0; i < length; i++)
        {
            int high = Character.digit(hex.charAt(i * 2), 16);
            int low = Character.digit(hex.charAt(i * 2 + 1), 16);
            int value = (high << 4) | low;
            raw[i] = (char)value;
        }
        return raw;
    }

    /**
     * シークレット生成
     * @param len Length of the string 文字列の長さ
     * @param typ Type of the string 0: Number only
     *                                1: Alphabet and Number
     *                                2: Alphabet, Number and special character
     * @return シークレット
     */
    public static String generateSecret(int len, int typ) {
        String secret = "";
        String base = SECRET_CHARS_TYP0;
        if (typ == 1) {
            base = SECRET_CHARS_TYP0 + SECRET_CHARS_TYP1;
        }
        else if (typ == 2) {
            base = SECRET_CHARS_TYP0 + SECRET_CHARS_TYP1 + SECRET_CHARS_TYP2;
        }
        char[] charset = base.toCharArray();
        for (int i = 0; i < len; i++) {
            secret += charset[new SecureRandom().nextInt(charset.length)];
        }
        return secret;
    }

    /**
     * ワンタイムパスワード生成
     * @param secret シークレット
     * @param pin ピン
     * @return ワンタイムパスワード
     */
    public static String generateOTP(String secret, String pin) {
        String otp = "";
        long movingFactor = ((((new Date()).getTime() / 10000) + ((TIMEZONE - 12) * 360)) / 6);    //epoch
        int codeDigits = 8;
        boolean addChecksum = false;
        int truncationOffset = -1;
        try {
            otp = OneTimePasswordAlgorithm.generateOTP((secret + pin).getBytes(), movingFactor, codeDigits, addChecksum, truncationOffset);
        } catch (NoSuchAlgorithmException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
        } catch (InvalidKeyException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
        }
        return otp;
    }

    /**
     * チェックURL
     * @param strUrl
     * @return
     */
    public static int checkUrl(String strUrl) {
        return checkFormat(PATTERN_URL, strUrl);
    }

    /**
     * チェックユーザID
     * @param strUserid
     * @return
     */
    public static int checkUserid(String strUserid) {
        return checkFormat(PATTERN_USERID, strUserid);
    }

    /**
     * 正規表現チェック
     * @param strPattern
     * @param strInput
     * @return 0:含みます 1:含みません
     */
    private static int checkFormat(String strPattern, String strInput) {
        Pattern p = Pattern.compile(strPattern, Pattern.CASE_INSENSITIVE);
        Matcher m = p.matcher(strInput);
        if (m.matches()) {
            return 0;
        }
        else {
            return 1;
        }
    }

    public static void useProxy(final String[] proxyinfo) {
        Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.CONFIG, "DEBUG - PROXY - {0} {1}:{2} {3}/{4}", new Object[]{proxyinfo[0], proxyinfo[1], proxyinfo[2], proxyinfo[3], proxyinfo[4]});

        //Proxy proxy = Proxy.NO_PROXY;

        // Do not user proxy
        if (null == proxyinfo[0] || "".equals(proxyinfo[0]) || "NONE".equals(proxyinfo[0])) {
            // RESET
            System.setProperty("http.proxyHost", "");
            System.setProperty("http.proxyPort", "");
            System.setProperty("https.proxyHost", "");
            System.setProperty("https.proxyPort", "");
            System.setProperty("http.proxyUser", "");
            System.setProperty("http.proxyPassword", "");

            System.setProperty("ftp.proxyHost", "");
            System.setProperty("ftp.proxyPort", "");
            System.setProperty("socksProxyHost", "");
            System.setProperty("socksProxyPort", "");
            System.setProperty("java.net.socks.username", "");
            System.setProperty("java.net.socks.password", "");
        }

        // Use proxy
        if ("HTTP".equals(proxyinfo[0])) {
            System.setProperty("http.proxyHost", proxyinfo[1]);
            System.setProperty("http.proxyPort", proxyinfo[2]); // default: 80
            System.setProperty("https.proxyHost", proxyinfo[1]);
            System.setProperty("https.proxyPort", proxyinfo[2]); // default: 443
            System.setProperty("http.proxyUser", proxyinfo[3]);
            System.setProperty("http.proxyPassword", proxyinfo[4]);
            //proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(proxyinfo[1], Integer.parseInt(proxyinfo[2])));
        }
        else if ("FTP".equals(proxyinfo[0])) {
            System.setProperty("ftp.proxyHost", proxyinfo[1]);
            System.setProperty("ftp.proxyPort", proxyinfo[2]); // default: 80
        }
        else if ("SOCKS".equals(proxyinfo[0])) {
            System.setProperty("socksProxyHost", proxyinfo[1]);
            System.setProperty("socksProxyPort", proxyinfo[2]); // default: 1080
            System.setProperty("java.net.socks.username", proxyinfo[3]);
            System.setProperty("java.net.socks.password", proxyinfo[4]);
            //proxy = new Proxy(Proxy.Type.SOCKS, new InetSocketAddress(proxyinfo[1], Integer.parseInt(proxyinfo[2])));
        }
        Authenticator.setDefault(new ProxyAuth(proxyinfo[3], proxyinfo[4]));

        //return proxy;
    }

    /**
     * データを指定URLにPOSTで送信
     * @param strUrl 指定URL
     * @param params 送信データ
     * @param proxyinfo プロキシ
     * @return サーバからの返り値
     */
    public static String postData(String strUrl, String params, String[] proxyinfo) {
        // Proxy Settings
        if (null != proxyinfo && proxyinfo.length > 0) {
            useProxy(proxyinfo);
        }

        // 登録処理を行う
        String errors = null;
        String response = null;
        HttpURLConnection conn = null;
        BufferedInputStream in = null;
        BufferedWriter writer = null;
        BufferedReader reader = null;
        //System.out.println("========== postData 1 ========== " + Logger.getLogger(SeciossUtils.LOG_LOGGERNAME));

        try {
            // サーバへPOSTで送信
            URL url = new URL(strUrl);

            if (strUrl.startsWith("https")) {
                // Trust all hostnames
                HostnameVerifier hostnameVerifier = new HostnameVerifier() {
                    @Override
                    public boolean verify(String hostname, SSLSession session) {
                        return true;
                    }
                };
                KeyManager[] keyManagers = null;
                // Trust all Server Certificates
                TrustManager[] trustManagers = null;
                KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
                trustStore.load(null);
                trustManagers = new TrustManager[] {new SeciossTrustManager(trustStore)};

                // Initiate Secure Socket Layer Context
                SSLContext sslContext = SSLContext.getInstance("TLS");
                sslContext.init(keyManagers, trustManagers, null);
                SSLSocketFactory sslSocketFactory = sslContext.getSocketFactory();

                //conn = (HttpsURLConnection) url.openConnection(proxy);
                conn = (HttpsURLConnection) url.openConnection();
                if (null != hostnameVerifier) {
                    // BUG in JAVA 8 https://bugs.openjdk.java.net/browse/JDK-8072464
                    // where if setHostnameVerifier(..) is used the SNI is not done from the client side.
                    //((HttpsURLConnection) conn).setHostnameVerifier(hostnameVerifier);
                }
                if (null != sslSocketFactory) {
                    ((HttpsURLConnection) conn).setSSLSocketFactory(sslSocketFactory);
                }
            }
            else if (strUrl.startsWith("http")) {
                //conn = (HttpURLConnection) url.openConnection(proxy);
                conn = (HttpURLConnection) url.openConnection();
            }
            else {
                Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "Invalid URL");
                return "99";
            }

            //System.out.println("========== postData 2 ========== " + Logger.getLogger(SeciossUtils.LOG_LOGGERNAME));
            conn.setRequestMethod("POST");
            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setConnectTimeout(30*1000);
            conn.setReadTimeout(30*1000);
            // write post data
            writer = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream()));
            writer.write(params);
            writer.flush();
            writer.close();
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.FINEST, "DEBUG - postData succeeded: [{0}] [{1}]", new Object[]{strUrl, params});
            //System.out.println("========== postData 3 ========== " + Logger.getLogger(SeciossUtils.LOG_LOGGERNAME));

            // サーバから返値取得
            int code = conn.getResponseCode();
            if (code == HttpURLConnection.HTTP_OK) {
                String encoding = "UTF-8";
                String contentType = conn.getContentType();
                if (null != contentType && !"".equals(contentType)) {
                    if (contentType.indexOf("charset=") > 0) {
                        encoding = contentType.substring(contentType.indexOf("charset=") + 8);
                    }
                }

                in = new BufferedInputStream(conn.getInputStream());
                reader = new BufferedReader(new InputStreamReader(in, encoding));
                StringBuilder buffer = new StringBuilder();
                String line = reader.readLine();
                while (line != null) {
                    buffer.append(line);
                    buffer.append("\n");
                    line = reader.readLine();
                }
                response = buffer.toString();
            }
            else {
                errors = conn.getResponseMessage();
            }
        }
        catch (Exception e) {
            errors = e.toString();
        }
        finally {
            if (conn != null) {
                conn.disconnect();
            }
            if (in != null) {
                try {
                    in.close();
                }
                catch (IOException e) {
                }
            }
            if (writer != null) {
                try {
                    writer.close();
                }
                catch (IOException e) {
                }
            }
            if (reader != null) {
                try {
                    reader.close();
                }
                catch (IOException e) {
                }
            }
            if (null != errors) {
                response = errors;
            }
        }

        return response;
    }

    /**
     * ローカルIPアドレス取得
     * @param testUrl 接続用URL
     * @param port 接続用ポート番号
     * @return ローカルIPアドレス
     */
    public static InetAddress getLocalIP(String testUrl, int port) {
        InetAddress localInetAddress;
        try {
            //useProxy(proxyinfo);
            Socket s = new Socket(testUrl, port);
            localInetAddress = s.getLocalAddress();
            if (localInetAddress.isLoopbackAddress()
                    || !localInetAddress.isSiteLocalAddress()
                    || "127.0.0.1".equals(localInetAddress.getHostAddress())) {
                localInetAddress = null;
            }
            s.close();
        }
        catch (IOException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get local Inet Address", ex);
            localInetAddress = null;
        }

        return localInetAddress;
    }


    /**
     * ローカルIPアドレス取得
     * @param testUrl 接続用URL
     * @param port 接続用ポート番号
     * @param proxyinfo
     * @return ローカルIPアドレス
     */
    public static InetAddress getLocalIP(String testUrl, int port, String[] proxyinfo) {
        // Proxy Settings
        if (null != proxyinfo && proxyinfo.length > 0) {
            useProxy(proxyinfo);
        }

        InetAddress localInetAddress;
        try {
            //useProxy(proxyinfo);
            Socket s = new Socket(testUrl, port);
            localInetAddress = s.getLocalAddress();
            if (localInetAddress.isLoopbackAddress()
                    || !localInetAddress.isSiteLocalAddress()
                    || "127.0.0.1".equals(localInetAddress.getHostAddress())) {
                localInetAddress = null;
            }
            s.close();
        } catch (UnknownHostException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get local Inet Address - UnknownHostException:\n{0}", ex);
            localInetAddress = null;
        } catch (IOException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get local Inet Address - IOException:\n{0}", ex);
            localInetAddress = null;
        }

        return localInetAddress;
    }

    public static String getLocalIpAddress() {
        try {
            for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
                NetworkInterface networkInterface = en.nextElement();
                for (Enumeration<InetAddress> enumIpAddr = networkInterface.getInetAddresses(); enumIpAddr.hasMoreElements();) {
                    InetAddress inetAddress = enumIpAddr.nextElement();
                    if (!inetAddress.isLoopbackAddress() && !inetAddress.isSiteLocalAddress()) {
                        return inetAddress.getHostAddress();
                    }
                    else if (!inetAddress.isLoopbackAddress() && inetAddress.isSiteLocalAddress()) {	//WIFI
                        return inetAddress.getHostAddress();
                    }
                }
            }
        } catch (SocketException ex) {
            return null;
        }
        return null;
    }

    /**
     * アクティブデバイスのMACアドレス取得
     * @param localInetAddress ローカルIPアドレス
     * @return 動作されているMACアドレス
     */
    public static String getMacaddress(InetAddress localInetAddress) {
        String macaddress = "";
        if (null == localInetAddress) {
            return macaddress;
        }

        Enumeration<NetworkInterface> devices;
        Enumeration<InetAddress> ipsOnEachDevice;
        NetworkInterface networkInterface;
        InetAddress inetAddress;
        int valid = 0;
        try {
            devices = NetworkInterface.getNetworkInterfaces();
            // get each device
            while (devices.hasMoreElements()) {
                networkInterface = devices.nextElement();

                //String deviceName = networkInterface.getDisplayName();

                byte[] mac = networkInterface.getHardwareAddress();
                if (//deviceName.indexOf("Virtual") == -1 &&
                                mac != null && networkInterface.isUp()
                                && !networkInterface.isLoopback()
                                && !networkInterface.isVirtual()) {

                    macaddress = String.format("%02X:%02X:%02X:%02X:%02X:%02X",
                                mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);

                    // skip blank mac address
                    if ("00:00:00:00:00:00".equals(macaddress)) {
                        macaddress = "";
                        continue;
                    }

                    ipsOnEachDevice = networkInterface.getInetAddresses();
                    while (ipsOnEachDevice.hasMoreElements()) {
                        inetAddress = ipsOnEachDevice.nextElement();
                        if (inetAddress instanceof Inet6Address) {
                            // IPv6 address
                        }
                        else {
                            // IPv4 address
                            if(localInetAddress.equals(inetAddress)) {
                                valid++;
                                break;
                            }
                        }
                    }

                    // 有効MACアドレス取得成功、処理終了
                    if (valid > 0) {
                        break;
                    }
                }

            }
        } catch (SocketException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get macaddress", ex);
        }
        return macaddress;
    }

    /**
     * MACアドレス取得
     * @return 動作されているMACアドレス
     */
    public static String getMacaddress() {
        String macaddress = "";
        // ローカルIP取得
        InetAddress localInetAddress;
        try {
            Socket s = new Socket("www.w3.org", 80);
            localInetAddress = s.getLocalAddress();
            if (localInetAddress.isLoopbackAddress()
                    || !localInetAddress.isSiteLocalAddress()
                    || "127.0.0.1".equals(localInetAddress.getHostAddress())) {
                localInetAddress = null;
            }
            s.close();
        }
        catch (IOException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, " failed to get local Inet Address", ex);
            localInetAddress = null;
        }
        if (null == localInetAddress) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.INFO, "failed to get local Inet Address");
            return macaddress;
        }

        Enumeration<NetworkInterface> devices;
        Enumeration<InetAddress> ipsOnEachDevice;
        NetworkInterface networkInterface;
        InetAddress inetAddress;
        int valid = 0;
        try {
            devices = NetworkInterface.getNetworkInterfaces();
            // get each device
            while (devices.hasMoreElements()) {
                networkInterface = devices.nextElement();

                //String deviceName = networkInterface.getDisplayName();

                byte[] mac = networkInterface.getHardwareAddress();
                if (//deviceName.indexOf("Virtual") == -1 &&
                                mac != null && networkInterface.isUp()
                                && !networkInterface.isLoopback()
                                && !networkInterface.isVirtual()) {

                    macaddress = String.format("%02X:%02X:%02X:%02X:%02X:%02X",
                                mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);

                    // skip blank mac address
                    if ("00:00:00:00:00:00".equals(macaddress)) {
                        macaddress = "";
                        continue;
                    }

                    ipsOnEachDevice = networkInterface.getInetAddresses();
                    while (ipsOnEachDevice.hasMoreElements()) {
                        inetAddress = ipsOnEachDevice.nextElement();
                        if (inetAddress instanceof Inet6Address) {
                            // IPv6 address
                        }
                        else {
                            // IPv4 address
                            if(localInetAddress.equals(inetAddress)) {
                                valid++;
                                break;
                            }
                        }
                    }

                    // 有効MACアドレス取得成功、処理終了
                    if (valid > 0) {
                        break;
                    }
                }

            }
        } catch (SocketException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get macaddress", ex);
        }
        Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.INFO, "macaddress: {0}", macaddress);
        return macaddress;
    }

    /**
     * ランダム文字列を生成する
     * @param length 文字列の長さ
     * @return ランダム文字列
     */
    public static String getRandomString(int length) {
        String str = "abcdefghijklmnopqrstuvwxyz0123456789";
        Random random = new Random();
        StringBuilder sb = new StringBuilder();
        int number = random.nextInt(36);
        sb.append(str.charAt(number));
        for (int i = 0; i < length-1; i++) {
            number = random.nextInt(36);
            sb.append(str.charAt(number));
        }
        Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "randstring: {0}", sb.toString());
        return sb.toString();
    }

    /**
     * デバイス名よりデバイスIDを取得
     * @param deviceName デバイス名以下より設定できます
     *     PhysicalMedia - ハードディスク
     *     DiskDrive - ハードディスク
     *     BaseBoard - マザーボード
     *     Processor - プロセッサ(CPU)
     *     BIOS - BIOS
     *     NetworkAdapter - MACアドレス
     * @return デバイスのユニークID
     */
    public static String getDeviceIDByName(String deviceName) {
        String result = "";
        String conditions = "";
        String deviceIDPropertyName;
        if (null == deviceName || "".equals(deviceName)) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.INFO, "device is unavailable: {0}", deviceName);
            return result;
        }
        else {
            if ("PhysicalMedia".equals(deviceName) || "DiskDrive".equals(deviceName)) {
                deviceIDPropertyName = "SerialNumber";
                conditions = "WHERE TAG = '\\\\\\\\.\\\\PHYSICALDRIVE0'";
            }
            else if ("BaseBoard".equals(deviceName) || "BIOS".equals(deviceName)) {
                deviceIDPropertyName = "SerialNumber";
            }
            else if ("Processor".equals(deviceName)) {
                deviceIDPropertyName = "ProcessorId";
            }
            else if ("NetworkAdapter".equals(deviceName)) {
                deviceIDPropertyName = "MACAddress";
                conditions = "WHERE MACAddress IS NOT NULL AND NetConnectionStatus = 2";
            }
            else {
                Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.INFO, "device unavailable: {0}", deviceName);
                return result;
            }
        }

        BufferedReader bufferedReader = null;
        Process process = null;
        try {
            File file = File.createTempFile("SeciossPass", ".vbs");
            file.deleteOnExit();
            BufferedWriter fw = new BufferedWriter(new FileWriter(file));

            StringBuilder sb = new StringBuilder();
            sb.append("MsgValue = \"\"").append("\r\n");
            sb.append("Set objWMIService = GetObject(\"winmgmts:\\\\.\\root\\cimv2\")").append("\r\n");
            sb.append("Set colItems = objWMIService.ExecQuery(\"Select * from Win32_").append(deviceName).append(" ").append(conditions).append("\", , 48)").append("\r\n");
            sb.append("For Each objItem in colItems").append("\r\n");
            sb.append("MsgValue = MsgValue & \" \" & objItem.").append(deviceIDPropertyName).append("\r\n");
            // デバイス複数存在しているの場合、最初のデバイスだけを取得する
            //sb.append("exit for").append("\r\n");
            sb.append("Next").append("\r\n");
            sb.append("Wscript.Echo MsgValue");

            fw.write(sb.toString());
            fw.close();
            process = Runtime.getRuntime().exec("cscript //NoLogo " + file.getPath());
            bufferedReader = new BufferedReader(new InputStreamReader(new BufferedInputStream(process.getInputStream())));
            String line = "";
            while (line != null) {
                result += line;
                line = bufferedReader.readLine();
            }
            file.delete();
        }
        catch (IOException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to create temp file", ex);
            result = "";
        }
        finally {
            try {
                if (null != bufferedReader) {
                    bufferedReader.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "DEBUG - Close bufferedReader failed", ex);
            }
            if (null != process) {
                process.destroy();
            }
        }

        result = result.trim();
        if ("".equals(result)) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.INFO, "failed to get device ID");
        }
        return result;
    }

    /**
     * ドライバのシリアル番号を取得
     * @param driveLetter ドライブに割り当てられた文字
     * @return ドライバのシリアル番号
     */
    public static String getDrivesSerialNumber(String driveLetter) {
        String result = "";
        BufferedWriter bufferedWriter = null;
        BufferedReader bufferedReader = null;
        Process process = null;
        try {
            File file = File.createTempFile("SeciossPass", ".vbs");
            file.deleteOnExit();
            bufferedWriter = new BufferedWriter(new FileWriter(file));

            StringBuilder sb = new StringBuilder();
            sb.append("MsgValue = \"\"").append("\r\n");
            sb.append("Set objFSO = CreateObject(\"Scripting.FileSystemObject\")").append("\r\n");
            sb.append("Set colItems = objFSO.GetDrive(\"").append(driveLetter).append(":\")").append("\r\n");
            sb.append("MsgValue = colItems.SerialNumber").append("\r\n");
            sb.append("Wscript.Echo MsgValue");

            bufferedWriter.write(sb.toString());

            process = Runtime.getRuntime().exec("cscript //NoLogo " + file.getPath());
            bufferedReader = new BufferedReader(new InputStreamReader(new BufferedInputStream(process.getInputStream())));
            String line = "";
            while (line != null) {
                result += line;
                line = bufferedReader.readLine();
            }
            file.delete();
        }
        catch (IOException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to create temp file", ex);
        }
        finally {
            try {
                if (null != bufferedWriter) {
                    bufferedWriter.close();
                }
                if (null != bufferedReader) {
                    bufferedReader.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "DEBUG - Close bufferedReader failed", ex);
            }
            if (null != process) {
                process.destroy();
            }
        }

        result = result.trim();
        if (result == null || result.length() < 1) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get Drive's serial number");
        }
        return result;
    }

    public static ArrayList<String> getIpconfigAll(String[] skipNicApp, String[] skipNicExclude, String[] skipNicUser) {
        //String strMacList = "";
        boolean skipNic = false;
        String[] arySkipNic = new String[skipNicApp.length + DEFALUT_SKIP_NIC.length];
        System.arraycopy(DEFALUT_SKIP_NIC, 0, arySkipNic, 0, DEFALUT_SKIP_NIC.length);
        System.arraycopy(skipNicApp, 0, arySkipNic, DEFALUT_SKIP_NIC.length, skipNicApp.length);

        String[] arySkipNicExclude = new String[skipNicExclude.length + DEFALUT_SKIP_NIC_EXCLUDE.length];
        System.arraycopy(DEFALUT_SKIP_NIC_EXCLUDE, 0, arySkipNicExclude, 0, DEFALUT_SKIP_NIC_EXCLUDE.length);
        System.arraycopy(skipNicExclude, 0, arySkipNicExclude, DEFALUT_SKIP_NIC_EXCLUDE.length, skipNicExclude.length);

        ArrayList<String> aryMacList = new ArrayList<String>();
        BufferedReader bufferedReader = null;
        Process process = null;
        try {
            Pattern p = Pattern.compile(PATTERN_IPCONFIG, Pattern.CASE_INSENSITIVE);
            process = Runtime.getRuntime().exec("ipconfig /all");
            bufferedReader = new BufferedReader(new InputStreamReader(new BufferedInputStream(process.getInputStream())));
            for (String line = bufferedReader.readLine(); line != null; line = bufferedReader.readLine()) {
                int skipExclude = 0;
                for (String strSkipNicExclude : arySkipNicExclude) {
                    if (!"".equals(strSkipNicExclude) && line.toLowerCase().contains(strSkipNicExclude.toLowerCase())) {
                        skipExclude++;
                        break;
                    }
                }
                if (skipExclude == 0) {
                    for (String strSkipNic : arySkipNic) {
                        if (!"".equals(strSkipNic) && line.toLowerCase().contains(strSkipNic.toLowerCase())) {
                            skipNic = true;
                            break;
                        }
                    }
                }

                Matcher m = p.matcher(line);
                if (m.find()) {
                    String mac = m.group(1);
                    // ユーザー無視するNIC
                    for (String strSkipNic : skipNicUser) {
                        if (!"".equals(strSkipNic) && line.toLowerCase().contains(strSkipNic.toLowerCase())) {
                            skipNic = true;
                            break;
                        }
                    }
                    if (!skipNic) {
                        if (!aryMacList.contains(mac)) {
                            //strMacList += ("".equals(strMacList) ? "" : "#") + mac;
                            aryMacList.add(mac);
                        }
                    }
                    skipNic = false;
                }
            }
            process.waitFor();
        }
        catch (IOException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get ipconfig all", ex);
        }
        catch (InterruptedException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get ipconfig all", ex);
        }
        finally {
            try {
                if (null != bufferedReader) {
                    bufferedReader.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "DEBUG - Close bufferedReader failed", ex);
            }
            if (null != process) {
                process.destroy();
            }
        }
        
        return aryMacList;
    }

    public static ArrayList<String> getIfconfig(String os, String[] skipNicApp, String[] skipNicExclude, String[] skipNicUser) {
        boolean skipNic = false;
        String[] arySkipNic = new String[skipNicApp.length + DEFALUT_SKIP_NIC.length];
        System.arraycopy(DEFALUT_SKIP_NIC, 0, arySkipNic, 0, DEFALUT_SKIP_NIC.length);
        System.arraycopy(skipNicApp, 0, arySkipNic, DEFALUT_SKIP_NIC.length, skipNicApp.length);

        String[] arySkipNicExclude = new String[skipNicExclude.length + DEFALUT_SKIP_NIC_EXCLUDE.length];
        System.arraycopy(DEFALUT_SKIP_NIC_EXCLUDE, 0, arySkipNicExclude, 0, DEFALUT_SKIP_NIC_EXCLUDE.length);
        System.arraycopy(skipNicExclude, 0, arySkipNicExclude, DEFALUT_SKIP_NIC_EXCLUDE.length, skipNicExclude.length);

        ArrayList<String> aryMacList = new ArrayList<String>();
        BufferedReader bufferedReader = null;
        Process process = null;
        try {
            String[] command;
            Pattern pattern;
            String patternValidMac = null;
            if ("Lnx".equals(os)) {
                command = new String[] {"sh", "-c", "LC_ALL=C $spass\\ifconfig"};
                pattern = Pattern.compile(PATTERN_IFCONFIG_LNX, Pattern.CASE_INSENSITIVE);
            }
            else {
                patternValidMac = "^\\s*options=[0-9a-z]+<([^>]+)>";
                command = new String[] {"sh", "-c", "LC_ALL=C $spass\\ifconfig"};
                pattern = Pattern.compile(PATTERN_IFCONFIG_MAC, Pattern.CASE_INSENSITIVE);
            }
            process = Runtime.getRuntime().exec(command);
            bufferedReader = new BufferedReader(new InputStreamReader(new BufferedInputStream(process.getInputStream())));
            for (String line = bufferedReader.readLine(); line != null; line = bufferedReader.readLine()) {
                int skipExclude = 0;
                for (String strSkipNicExclude : arySkipNicExclude) {
                    if (!"".equals(strSkipNicExclude) && line.toLowerCase().contains(strSkipNicExclude.toLowerCase())) {
                        skipExclude++;
                        break;
                    }
                }
                if (skipExclude == 0) {
                    for (String strSkipNic : arySkipNic) {
                        if (!"".equals(strSkipNic) && line.toLowerCase().contains(strSkipNic.toLowerCase())) {
                            skipNic = true;
                            break;
                        }
                    }
                }
                String options = null;
                if (patternValidMac != null) {
                    Pattern p = Pattern.compile(patternValidMac, Pattern.CASE_INSENSITIVE);
                    Matcher m = p.matcher(line);
                    if (m.find()) {
                        options = m.group(1);
                        //System.out.println("skipNic:" + skipNic + " | options: " + options);
                        if (options != null && !options.contains("RXCSUM")) {
                            skipNic = true;
                        }
                    }
                }

                Matcher matcher = pattern.matcher(line);
                if (matcher.find()) {
                    String mac = matcher.group(1);
                    //System.out.println("skipNic:" + skipNic + " | mac: " +  mac);
                    // ユーザー無視するNIC
                    for (String strSkipNic : skipNicUser) {
                        if (!"".equals(strSkipNic) && line.toLowerCase().contains(strSkipNic.toLowerCase())) {
                            skipNic = true;
                            break;
                        }
                    }

                    if (!skipNic) {
                        if (!aryMacList.contains(mac)) {
                            aryMacList.add(mac);
                        }
                    }
                    skipNic = false;
                }
            }
            process.waitFor();
        }
        catch (IOException | InterruptedException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to get ifconfig all", ex);
        }
        finally {
            try {
                if (null != bufferedReader) {
                    bufferedReader.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "DEBUG - Close bufferedReader failed", ex);
            }
            if (null != process) {
                process.destroy();
            }
        }

        return aryMacList;
    }

    public static ArrayList<String> sortArrayList(ArrayList<String> target) {
        Collections.sort(target, new Comparator<String>() {
            @Override
            public int compare(String o1, String o2) {
                return o1.compareTo(o2);
            }
        });

        return target;
    }

    public static ArrayList<String> str2Ary(String strMacList) {
        if (null == strMacList || "".equals(strMacList)) {
            return null;
        }
        ArrayList<String> aryMacList = new ArrayList<String>();
        String[] macList = strMacList.split("#");
        aryMacList.addAll(Arrays.asList(macList));

        return aryMacList;
    }

    private static class ProxyAuth extends Authenticator {
        private PasswordAuthentication auth;

        private ProxyAuth(String user, String password) {
            auth = new PasswordAuthentication(user, password == null ? new char[]{} : password.toCharArray());
        }

        @Override
        protected PasswordAuthentication getPasswordAuthentication() {
            return auth;
        }
    }

    public static void copyFileUsingFileStreams(File source, File dest) {
        InputStream input = null;
        OutputStream output = null;
        try {
            input = new FileInputStream(source);
            output = new FileOutputStream(dest);
            byte[] buf = new byte[1024];
            int bytesRead = input.read(buf);
            while (bytesRead > 0) {
                output.write(buf, 0, bytesRead);
                bytesRead = input.read(buf);
            }
        }
        catch (FileNotFoundException ex) {
                Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "file not exist: " + source + ", " +dest, ex);
        }
        catch (IOException ex) {
                Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "copyFileUsingFileStreams IOException", ex);
        }
        finally {
            try {
                if (null != input) {
                    input.close();
                }
                if (null != output) {
                    output.close();
                }
            }
            catch (IOException ex) {
                    Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "copyFileUsingFileStreams IOException", ex);
            }
        }
    }

    /**
     * encode a string with given encoding method
     * @param secretdecode encoding method
     * @param Encode original string
     * @return encoded string of original
     */
    public static String encodeString(AES_PBE_BC secretdecode, String Encode) {
        String tmpStr = "";
        try {
            tmpStr = secretdecode.encrypt(Encode);
        } catch(Exception ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "encode error", ex);
        }
        return tmpStr;
    }

    /**
     * decode a string with given encoding method
     * @param secretdecode encoding method
     * @param Encoded encoded string
     * @return decoded (original) string
     */
    public static String decodeString(AES_PBE_BC secretdecode, byte[] Encoded) {
        String tmpStr = "";
        try {
            tmpStr = secretdecode.decrypt(new String(Encoded));
        }
        catch(InvalidCipherTextException ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to decode string: wrong secret", ex);
            return tmpStr;
        }
        catch(Exception ex) {
            Logger.getLogger(SeciossUtils.LOG_LOGGERNAME).log(Level.SEVERE, "failed to decode string: unkonw error", ex);
            return tmpStr;
        }

        if (tmpStr.length() <= 0) {
            tmpStr = new String(Encoded);
            tmpStr = new MD5(tmpStr).asHex();
        }

        if (tmpStr.indexOf((char)0) > 0) {
            tmpStr = tmpStr.substring(0, tmpStr.indexOf((char)0));
        }

        return tmpStr;
    }

}
