/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package jp.co.secioss.lib;

import java.awt.Component;
import java.awt.Container;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Properties;
import java.util.Set;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import javax.swing.UIManager;

/**
 *
 * @author shuzx
 */
public class SeciossUtils {

    public static final int LOG_HANDLER_FILE = 0;
    public static final int LOG_HANDLER_CONSOLE = 1;
    public static final String LOG_LOGGERNAME = "slinkpass.logger";

    public static final String DEFAULT_RCSTATUS_TIMEMIN = "10";
    public static final String DEFAULT_SERVER_NAME = "slink.secioss.com";
    public static final String DEFAULT_PORTAL_NAME = "SeciossLink Portal";
    public static final String DEFAULT_BID_STRING = "computer";

    // property names of SeciossPass.properties
    public static final String PROPERTY_LOG_LEVEL = "log";
    public static final String PROPERTY_SALT = "salt";
    public static final String PROPERTY_AUTH_URL = "authurl";
    public static final String PROPERTY_SECRET = "secret";
    public static final String PROPERTY_DEVICE_TYPE = "device";
    public static final String PROPERTY_DEVICE_INFO = "deviceinfo";
    public static final String PROPERTY_DEVICE_SKIP = "deviceskip";
    public static final String PROPERTY_DEVICE_SKIP_EXCLUDE = "deviceskipexclude";
    public static final String PROPERTY_USERID = "userid";
    public static final String PROPERTY_RCSTATUS_TIMEMIN = "rcstatus";
    public static final String PROPERTY_SERVER_NAME = "portal";
    public static final String PROPERTY_PORTAL_NAME = "portal";
    public static final String PROPERTY_VISIBLE_RCPARAM = "v_rc_param";

    // property names of secioss.properties
    public static final String PROPERTY_ACCESS_URL = "accessurl";
    public static final String PROPERTY_BROWSER_PATH = "browser";
    public static final String PROPERTY_IGNORE_NIC = "ignorenic";
    public static final String PROPERTY_PROXY_TYPE = "proxytype";
    public static final String PROPERTY_PROXY_HOST = "proxyhost";
    public static final String PROPERTY_PROXY_PORT = "proxyport";
    public static final String PROPERTY_PROXY_USER = "proxyuser";
    public static final String PROPERTY_PROXY_PASS = "proxypass";

    // filenames of properties files
    public static final String PROPERTIES_SECRET_INFOS_FILENAME_WIN = ".secioss.properties";
    public static final String PROPERTIES_SECRET_INFOS_FILENAME_MAC = ".secioss.properties";
    public static final String PROPERTIES_GENERAL_SETTINGS_FILENAME = "secioss.properties";

    public static final String URI_SCHEME = "https://";
    public static final String URI_PATH_SECRET = "/api/spass.php";            // Register account
    public static final String URI_PATH_SIGNON = "/pub/computerlogin.cgi";    // Web Client Login
    public static final String URI_PATH_SCHEME = "/api/prerequisite.php";     // Web Client restriction
    public static final String URI_PATH_IPAUTH = "/api/registerip.php";       // Rich Client Login
    public static final String URI_PATH_APPXML = "/api/applist.xml";          // Rich Client restriction
    public static final String URI_PATH_CREDIT = "/api/credential.php";       // VPN Client Login
    public static final String URI_PATH_PORTAL = "/user/";                    // Portal Site

    public static final String[] ERROR_CODES = new String[] {"1", "2", "3", "4", "-1", "-2", "97", "98", "99"};

    public static void center(Container parent, Component comp) {
        Dimension compSize = comp.getSize();
        Rectangle parentBounds;
        if ((parent == null) || (parent.getBounds().width < compSize.width) ||
            (parent.getBounds().height < compSize.height)) {
            parentBounds = new Rectangle(comp.getToolkit().getScreenSize());
            parentBounds.setLocation(0, 0);
        }
        else {
            parentBounds = parent.getBounds();
        }

        int x = parentBounds.x + (parentBounds.width / 2 - compSize.width / 2);
        int y = parentBounds.y + (parentBounds.height / 2 - compSize.height / 2);
        comp.setLocation(x, y);
    }

    public static Logger initLoggerHandler(int type, int level, String logFilename) {
        Logger logger = Logger.getLogger(LOG_LOGGERNAME);

        // Set log format
        Formatter formatter = new Formatter() {
            @Override
            public String format(LogRecord record) {
                StringBuilder sb = new StringBuilder();
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                String time = sdf.format(new Date());
                sb.append(time);
                sb.append(" ");
                sb.append(record.getSourceClassName());
                sb.append(".");
                sb.append(record.getSourceMethodName());
                sb.append(" ");
                sb.append(record.getLevel());
                sb.append(" - ");
                sb.append(formatMessage(record));
                sb.append("\n");
                return sb.toString();
            }
        };

        if (type == 0 && level > 0) {
            try {
                Handler filehandler;
                filehandler = new FileHandler(logFilename, 500000, 1, true);
                filehandler.setFormatter(formatter);
                logger.addHandler(filehandler);
            } catch (IOException ex) {
                System.out.println(ex);
            }
        }

        Handler consoleHandler = new ConsoleHandler();
        consoleHandler.setFormatter(formatter);

        // Set log level
        // Both the logger and its handlers have Log Levels
        switch (level) {
            case 0:
                consoleHandler.setLevel(Level.OFF);
                logger.setLevel(Level.OFF);
                break;
            case 1:
                consoleHandler.setLevel(Level.SEVERE);
                logger.setLevel(Level.SEVERE);
                break;
            case 2:
                consoleHandler.setLevel(Level.WARNING);
                logger.setLevel(Level.WARNING);
                break;
            case 3:
                consoleHandler.setLevel(Level.INFO);
                logger.setLevel(Level.INFO);
                break;
            case 4:
                consoleHandler.setLevel(Level.CONFIG);
                logger.setLevel(Level.CONFIG);
                break;
            case 5:
                consoleHandler.setLevel(Level.FINE);
                logger.setLevel(Level.FINE);
                break;
            case 6:
                consoleHandler.setLevel(Level.FINER);
                logger.setLevel(Level.FINER);
                break;
            case 7:
                consoleHandler.setLevel(Level.FINEST);
                logger.setLevel(Level.FINEST);
                break;
            case 8:
                consoleHandler.setLevel(Level.ALL);
                logger.setLevel(Level.ALL);
                break;
            default:
                consoleHandler.setLevel(Level.OFF);
                logger.setLevel(Level.OFF);
        }
        logger.addHandler(consoleHandler);

        logger.log(Level.FINEST, "Initialise Logger Succeeded - Level: {0}", level);
        return logger;
    }

    public static String readProperties(String key, String defaultValue, String path) {
        String value = null;
        Properties props = new Properties();
        InputStream in = null;
        try {
            in = new BufferedInputStream(new FileInputStream(path));
            props.load(in);
            value = props.getProperty(key, defaultValue);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.WARNING, "{0} {1} is not found", new String[] {key, path});
        } catch (IOException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, path + " is fail to read", ex);
        }
        finally {
            try {
                if (null != in)  {
                    in.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
            }
        }
        if (null == value || "".equals(value)) {
            value = defaultValue;
        }
        return value;
    }

    public static int writeProperties(String[] keys, String[] values, String path) {
        if (keys.length != values.length) {
            return -1;
        }
        Properties props = new Properties();
        InputStream in = null;
        OutputStream out = null;
        try {
            in = new BufferedInputStream(new FileInputStream(path));
            props.load(in);
        }
        catch (FileNotFoundException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.WARNING, "{0} {1} is not found, create a new one(multiple)", new String[] {Arrays.toString(keys), path});
            //return 1;
        }
        catch (IOException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, "{0} {1} is fail to read(multiple)", new String[] {Arrays.toString(keys), path});
            return 2;
        }
        finally {
            try {
                if (null != in)  {
                    in.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
            }
        }

        try {
            out = new BufferedOutputStream(new FileOutputStream(path));
            for(int i = 0; i < keys.length; i++) {
                props.setProperty(keys[i], values[i]);
            }
            props.store(out, "Last update at ");

            // シークレットファイルを隠しする
            if (!path.equals(PROPERTIES_GENERAL_SETTINGS_FILENAME)) {
                //setHidden(null, path, true);
            }
        }
        catch (IOException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, "{0} {1} is fail to write(multiple)", new String[] {Arrays.toString(keys), path});
            return 2;
        }
        finally {
            try {
                if (null != out)  {
                    out.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
            }
        }

        return 0;
    }

    public static int writeProperties(String key, String value, String path) {
        Properties props = new Properties();
        InputStream in = null;
        OutputStream out = null;
        try {
            in = new BufferedInputStream(new FileInputStream(path));
            props.load(in);
        }
        catch (FileNotFoundException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.WARNING, "{0} {1} is not found, create a new one(single)", new String[] {key, path});
            //return 1;
        }
        catch (IOException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, "{0} {1} is fail to read(single)", new String[] {key, path});
            return 2;
        }
        finally {
            try {
                if (null != in)  {
                    in.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
            }
        }

        try {
            out = new BufferedOutputStream(new FileOutputStream(path));
            props.setProperty(key, value);
            props.store(out, "Last update '" + key + "' at ");

            // シークレットファイルを隠しする
            if (!path.equals(PROPERTIES_GENERAL_SETTINGS_FILENAME)) {
                //setHidden(null, path, true);
            }
        }
        catch (IOException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, path + " is fail to write(single) " + key, ex);
            return 2;
        }
        finally {
            try {
                if (null != out)  {
                    out.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
            }
        }

        return 0;
    }

    /**
     * OSがWindowsの場合、ファイルの隠し属性を設定する
     * @param directory 設定先ファイルの場所
     * @param filename 設定先ファイルの名前
     * @param isHidden 隠しフラグ
     */
    public static void setHidden(String directory, String filename, boolean isHidden) {
        if (null == filename || "".equals(filename)) {
            return;
        }

        // Windowsのみ対応しています
        String os = System.getProperty("os.name");
        if (!os.startsWith("Windows")) {
            return;
        }

        String filepath;
        if (null == directory || "".equals(directory)) {
            File dir = new File(".");
            String absolutePath = dir.getAbsolutePath();
            absolutePath = absolutePath.substring(0, absolutePath.length()-1);
            filepath = absolutePath + filename;
        }
        else {
            filepath = directory + filename;
        }

        File file = new File(filepath);
        if (!file.exists()) {
            return;
        }

        // In Java 7
        //Path file = FileSystems.getDefault().getPath("/j", "sa");;
        //Files.setAttribute(path, "dos:hidden", true);

        // In Java 6
        String cmd;
        if (isHidden) {
            cmd = "attrib +h " + filepath;
        }
        else {
            cmd = "attrib -h " + filepath;
        }
        try {
            Runtime.getRuntime().exec(cmd);
        } catch (IOException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.WARNING, filepath + " is fail to set hidden", ex);
        }
    }

    /**
     * 設定属性を削除
     * @param key 設定属性のキー
     * @param path 設定ファイルのファイル名
     * @return 0:成功 0以外はエラーコード
     */
    public static int removeProperties(String key, String path) {
        Properties props = new Properties();
        InputStream in = null;
        OutputStream out = null;
        try {
            in = new BufferedInputStream(new FileInputStream(path));
            props.load(in);
        }
        catch (FileNotFoundException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.WARNING, path + " is not found, no need to remove", ex);
            return 1;
        }
        catch (IOException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, path + " is fail to read(single)", ex);
            return 2;
        }
        finally {
            try {
                if (null != in)  {
                    in.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
            }
        }

        try {
            out = new BufferedOutputStream(new FileOutputStream(path));
            props.remove(key);
            props.store(out, "Last update '" + key + "' at ");

            // シークレットファイルを隠しする
            if (!path.equals(PROPERTIES_GENERAL_SETTINGS_FILENAME)) {
                //setHidden(null, path, true);
            }
        }
        catch (IOException ex) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, path + " is fail to write(single)", ex);
            return 2;
        }
        finally {
            try {
                if (null != out)  {
                    out.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(LOG_LOGGERNAME).log(Level.SEVERE, null, ex);
            }
        }

        return 0;
    }

    public static String getSecretPropertiesFilename() {
        String secretPropertiesFilename;
        String os = System.getProperty("os.name");
        if (os.startsWith("Windows")) {
            secretPropertiesFilename = PROPERTIES_SECRET_INFOS_FILENAME_WIN;
        }
        else if (os.startsWith("Mac")) {
            secretPropertiesFilename = PROPERTIES_SECRET_INFOS_FILENAME_MAC;
        }
        else {
            secretPropertiesFilename = PROPERTIES_SECRET_INFOS_FILENAME_MAC;
        }
       return secretPropertiesFilename;
    }

    /**
     * 端末IDを取得
     * 優先順：MACアドレス、ドライブシリアル、ディスクシリアル
     * @param device 端末種類
     * @param oldDeviceData 既存の端末情報
     * @param ignoreNic 無視するNIC情報
     * @return 文字列配列{端末ID, 端末種類}
     */
    public static String [] getDeviceID(String device, String oldDeviceData) {
        String deviceID = "";
        String deviceType = "ipconfig";
        String deviceData = "";
        if (null != device && !"".equals(device)) {
            deviceType = device;
        }

        // ipconfig コマンドから端末IDを取得
        if ("ipconfig".equals(deviceType)) {
            // 保存しているMacリスト
            String strSkip = SeciossUtils.readProperties(SeciossUtils.PROPERTY_DEVICE_SKIP, "", getSecretPropertiesFilename());
            String strSkipExclude = SeciossUtils.readProperties(SeciossUtils.PROPERTY_DEVICE_SKIP_EXCLUDE, "", getSecretPropertiesFilename());
            String orgStrMacList;
            if (null == oldDeviceData) {
                orgStrMacList = SeciossUtils.readProperties(SeciossUtils.PROPERTY_DEVICE_INFO, "", getSecretPropertiesFilename());
            }
            else {
                orgStrMacList = oldDeviceData;
            }
            ArrayList<String> orgAryMacList = SeciossCommon.str2Ary(orgStrMacList);

            String ignoreNic = SeciossUtils.readProperties(SeciossUtils.PROPERTY_IGNORE_NIC, "", SeciossUtils.PROPERTIES_GENERAL_SETTINGS_FILENAME);

            // 新しMacリスト
            ArrayList<String> aryMacList;
            String os = System.getProperty("os.name");
            if (os.startsWith("Windows")) {
                aryMacList = SeciossCommon.sortArrayList(SeciossCommon.getIpconfigAll(strSkip.split(","), strSkipExclude.split(","), ignoreNic.split(",")));
            }
            else if (os.startsWith("Mac")) {
                aryMacList = SeciossCommon.sortArrayList(SeciossCommon.getIfconfig("Mac", strSkip.split(","), strSkipExclude.split(","), ignoreNic.split(",")));
            }
            else {
                aryMacList = SeciossCommon.sortArrayList(SeciossCommon.getIfconfig("Lnx", strSkip.split(","), strSkipExclude.split(","), ignoreNic.split(",")));
                if (aryMacList.isEmpty()) {
                    aryMacList = SeciossCommon.sortArrayList(SeciossCommon.getIfconfig("Mac", strSkip.split(","), strSkipExclude.split(","), ignoreNic.split(",")));
                }
            }

            boolean isOrg = true;
            if (null != orgAryMacList) {
                for (String orgMac : orgAryMacList) {
                    if (!aryMacList.contains(orgMac)) {
                        isOrg = false;
                        break;
                    }
                }
            }

            // 取得したMACリストと保存しているMACリストが一致
            if (isOrg && !"".equals(orgStrMacList)) {
                deviceData = orgStrMacList;
                deviceID = new MD5(orgStrMacList).asHex();
                Logger.getLogger(LOG_LOGGERNAME).log(Level.FINE, "Same deviceID {0} {1} {2} {3}", new String [] {os, deviceType, deviceID, deviceData});
            }
            else {
                String strMacList = "";
                for (String strMac : aryMacList) {
                    strMacList += ("".equals(strMacList) ? "" : "#") + strMac;
                }
                if (!"".equals(strMacList)) {
                    deviceData = strMacList;
                    deviceID = new MD5(strMacList).asHex();
                }
                Logger.getLogger(LOG_LOGGERNAME).log(Level.CONFIG, "New deviceID {0} {1} {2} {3}", new String [] {os, deviceType, deviceID, deviceData});
            }
        }
        else {
            // まずMACアドレスを取りに行きます
            String testUrl = SeciossUtils.readProperties(SeciossUtils.PROPERTY_AUTH_URL, "www.secioss.co.jp", getSecretPropertiesFilename());
            int port = 443;
            if (testUrl.startsWith("https://")) {
                testUrl = testUrl.substring(8);
                port = 443;
            }
            else if (testUrl.startsWith("http://")) {
                testUrl = testUrl.substring(7);
                port = 80;
            }
            InetAddress localInetAddress = SeciossCommon.getLocalIP(testUrl, port);
            String macaddress = SeciossCommon.getMacaddress(localInetAddress);
            //macaddress = "FF:FF:FF:FF:FF:FF";//test用

            if ("MACAddress".equals(deviceType) && !"".equals(macaddress)) {
                deviceID = macaddress;
            }
            // MACアドレスが取られないの場合、ドライブ又はディスクシリアルを取ります
            else {
                // ドライブシリアル
                //String driveLetter = System.getProperty("user.dir");
                //driveLetter = driveLetter.substring(0,1);
                //deviceID = SeciossCommon.getDrivesSerialNumber(driveLetter);

                // ディスクシリアル
                deviceType = "PhysicalMedia";    //"DiskDrive"
                deviceID = SeciossCommon.getDeviceIDByName(deviceType);
            }
            Logger.getLogger(LOG_LOGGERNAME).log(Level.INFO, "deviceID for {0}: {1} {2}", new String [] {deviceType, deviceID, deviceData});
        }

        if ("".equals(deviceID)) {
            Logger.getLogger(LOG_LOGGERNAME).log(Level.INFO, "failed to get deviceID of all kind");
            return new String [] {"", ""};
        }

        return new String [] {deviceID, deviceType, deviceData};
    }

    public static void launchBrowser(String browser, String strUrl) throws IOException, URISyntaxException {
        if (null == strUrl || "".equals(strUrl) || SeciossCommon.checkUrl(strUrl) != 0) {
            throw new MalformedURLException("Invalid URL: " + strUrl);
        }

        // open default browser using Desktop API if browser settings is not set
        if (null == browser || "".equals(browser)) {
            Desktop desktop = Desktop.getDesktop();
            if (Desktop.isDesktopSupported() && desktop.isSupported(Desktop.Action.BROWSE)) {
                URI uri = new URI(strUrl);
                desktop.browse(uri);
            }
        }
        else {
            // "C:\Program Files (x86)\Mozilla Firefox\firefox.exe" -no-remote -P default
            String fullpath;
            String param = "";
            if (browser.indexOf("\"") == 0 && browser.lastIndexOf("\"") > 1) {
                fullpath = browser.substring(1);
                fullpath = fullpath.substring(0, fullpath.indexOf("\""));
                if (!isExecutable(fullpath)) {
                    throw new FileNotFoundException("Invalid browser: " + fullpath);
                }
                if (browser.contains("\" ") && browser.indexOf("\" ") > 1) {
                    param = browser.substring(browser.indexOf("\" ") + 2);
                }
            }
            else {
                fullpath = browser;
                if (!isExecutable(fullpath)) {
                    // Since iexplore is not in env PATH, we cannot check file exists in that way
                    //throw new FileNotFoundException("Invalid browser: " + fullpath);
                }
            }

            String os = System.getProperty("os.name");
//            String[] cmd;
            if (os.startsWith("Windows")) {
//                String[] tmpcmd = new String[] {"cmd", "/c", "start", " ", fullpath};
//                Object[] params = parseParams(param + " \"" + strUrl + "\"");
//                cmd = new String[tmpcmd.length + params.length];
//                System.arraycopy(tmpcmd, 0, cmd, 0, tmpcmd.length);
//                System.arraycopy(params, 0, cmd, tmpcmd.length, params.length);
//                Runtime.getRuntime().exec(cmd);
                launchApplication(fullpath, param + " \"" + strUrl + "\"", false);
            }
            else if (os.startsWith("Mac")) {
//                String[] tmpcmd = new String[] {"open", "-a", fullpath};
//                Object[] params = parseParams("".equals(param) ? strUrl : param + " " + strUrl);
//                cmd = new String[tmpcmd.length + params.length];
//                System.arraycopy(tmpcmd, 0, cmd, 0, tmpcmd.length);
//                System.arraycopy(params, 0, cmd, tmpcmd.length, params.length);
//                Runtime.getRuntime().exec(cmd);
                launchApplication(fullpath, "\"" + strUrl + "\" " + param, false);
            }
            else {
//                String[] tmpcmd = new String[] {fullpath};
//                Object[] params = parseParams("".equals(param) ? strUrl : param + " " + strUrl);
//                cmd = new String[tmpcmd.length + params.length];
//                System.arraycopy(tmpcmd, 0, cmd, 0, tmpcmd.length);
//                System.arraycopy(params, 0, cmd, tmpcmd.length, params.length);
//                Runtime.getRuntime().exec(cmd);
                //launchApplication(fullpath, param + " \"" + strUrl + "\"", false);
                launchApplication(fullpath, param + " " + strUrl, false);
            }

//            Logger.getLogger(LOG_LOGGERNAME).log(Level.INFO, "DEBUG - Launch browser command: {0}", testCmd(cmd));
        }
    }

    /**
     * @param name May program executable
     * @return
     */
    public static boolean isExecutable(String name) {
        String os = System.getProperty("os.name");
        String extension = "";
        if (os.startsWith("Windows")) {
            extension = ".exe";
        }
        else if (os.startsWith("Mac")) {
            extension = ".app";
        }

        for (String dir : System.getenv("PATH").split(";")) {
            if (new File(dir + name).exists() || new File(dir + name + extension).exists()) {
                return true;
            }
        }
        return new File(name).exists();
    }

    public static String parseFilename(String fullpath) {
        String filename;
        if (null == fullpath || "".equals(fullpath)) {
            return "";
        }
        else {
            filename = fullpath.toLowerCase();
        }
        String os = System.getProperty("os.name");
        if (os.startsWith("Windows")) {
            filename = filename.substring(filename.lastIndexOf("\\") + 1);
        }
        else {
            filename = filename.substring(filename.lastIndexOf("/") + 1);
        }
        return filename;
    }

    public static Object[] parseParams(String param) {
        ArrayList<String> alparams = new ArrayList<>();
        String processed;
        String remaining = param;
        while (remaining.contains("\"") || remaining.contains(" ")) {
            if (remaining.indexOf("\"") == 0) {
                remaining = remaining.substring(1);
                if (remaining.contains("\"")) {
                    processed = "\"" + remaining.substring(0, remaining.indexOf("\"")) + "\"";
                    remaining = remaining.substring(remaining.indexOf("\"")+1).trim();
                }
                else {
                    continue;
                }
            }
            else {
                if (remaining.contains(" ")) {
                    processed = remaining.substring(0, remaining.indexOf(" "));
                    remaining = remaining.substring(remaining.indexOf(" ")+1);
                }
                else {
                    break;
                }
            }
            alparams.add(processed);
        }
        if (!"".equals(remaining)) {
            alparams.add(remaining);
        }
        Object[] params = alparams.toArray();
        return params;
    }

    public static void launchApplication(String fullpath, String param, boolean verify) throws IOException {
        File file = new File(fullpath);
        String os = System.getProperty("os.name");
        if (verify || "".equals(param)) {
            if (os.startsWith("Mac")) {
                if (!file.exists()) {
                    throw new FileNotFoundException("Invalid application path(" + fullpath + ")");
                }
            }
            else {
                if (!file.exists() || !file.isFile()) {
                    throw new FileNotFoundException("Invalid application path(" + fullpath + ")");
                }
            }
        }

        if (null == param || "".equals(param)) {
            Desktop desktop = Desktop.getDesktop();
            desktop.open(file);
        }
        else {
            String cmd;
            if (os.startsWith("Windows")) {
                cmd = "cmd /c start \"\" \"" + fullpath + "\" " + param;
                Runtime.getRuntime().exec(cmd);
            }
            else if (os.startsWith("Mac")) {
                cmd = "open" + " -a \"" + fullpath + "\" " + param;
                Runtime.getRuntime().exec(new String[]{"/bin/sh", "-c", cmd});
            }
            else {
                cmd = fullpath + " " + param;
                Runtime.getRuntime().exec(cmd);
            }
            Logger.getLogger(LOG_LOGGERNAME).log(Level.INFO, "DEBUG - launchApplication: {0} ", new String[]{cmd});
        }
    }

    /*
     * 未使用
     */
    public static String runCommand(String cmd) throws IOException, InterruptedException {
        String result = "";
        if (null == cmd || "".equals(cmd)) {
            throw new IOException("Invalid command");
        }
        String os = System.getProperty("os.name");
        Process process;
        if (os.startsWith("Windows")) {
            process = Runtime.getRuntime().exec("cmd /c " + cmd, null);
        }
        else if (os.startsWith("Mac")) {
            process = Runtime.getRuntime().exec(cmd, null);
        }
        else {
            process = Runtime.getRuntime().exec(cmd, null);
        }
        process.waitFor();
        try (BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(process.getInputStream(), System.getProperty("sun.jnu.encoding")))) {
            String line = bufferedReader.readLine();
            while (line != null) {
                if (!"".equals(line)) {
                    result += line + "\n";
                }
                line = bufferedReader.readLine();
            }
        }
        process.destroy();
        
        return result;
    }
    
    public static void setDefaultFontSize(int size) {
        Set<Object> keySet = UIManager.getLookAndFeelDefaults().keySet();
        Object[] keys = keySet.toArray(new Object[keySet.size()]);
        for (Object key : keys) {
            if (key != null && key.toString().toLowerCase().contains("font")) {
                //System.out.println(key);
                Font font = UIManager.getDefaults().getFont(key);
                if (font != null) {
                    font = font.deriveFont((float)size);
                    UIManager.put(key, font);
                }
            }
        }
    }

}
